#!/bin/bash

HOSTAPD_CONF="/etc/hostapd/hostapd.conf"
INTERFACES="/etc/network/interfaces"
DNSMASQ_CONF="/etc/dnsmasq.conf"
DHCPCD_CONF="/etc/dhcpcd.conf"
WPA_CONF="/etc/wpa_supplicant/wpa_supplicant.conf"
DECONZ_CONF_DIR="/home/pi/.local/share"
ZLLDB=""

AP_NAME=""   # empty string | Not set | value
AP_PW=""     # empty string | value

LOG_EMERG="<0>"
LOG_ALERT="<1>"
LOG_CRIT="<2>"
LOG_ERROR="<3>"
LOG_WARN="<4>"
LOG_NOTICE="<5>"
LOG_INFO="<6>"
LOG_DEBUG="<7>"

# format of sqlite3 query result:
# <key>|<value>

OLD_AP_NAME=""
OLD_AP_PW=""

function checkWifiConfig {

	# is sqlite3 installed?
	sqlite3 --version &> /dev/null
	if [ $? -ne 0 ]; then
		echo "${LOG_WARN}sqlite3 not installed"
		return
	fi

	# look for latest config in specific order
	drs=("data/dresden-elektronik/deCONZ/zll.db" "dresden-elektronik/deCONZ/zll.db" "deCONZ/zll.db")
	for i in "${drs[@]}"; do
		if [ -f "${DECONZ_CONF_DIR}/$i" ]; then
			ZLLDB="${DECONZ_CONF_DIR}/$i"
			break
		fi
	done

	if [ ! -f "$ZLLDB" ]; then
		# might have been deleted
		ZLLDB=""
	fi

	if [[ -z "$ZLLDB" ]]; then
		echo "${LOG_WARN}no database file (zll.db) found"
		return
	fi

	params=( [0]="wifi" [1]="wifiname" [2]="wifiappw" [3]="wifitype" )
	values=()

	for i in {0..3}; do
		param=${params[$i]}
		value=$(sqlite3 $ZLLDB "select * from config2 where key=\"${param}\"")
		if [ $? -ne 0 ]; then
			return
		fi

		value=$(echo $value | cut -d'|' -f2)

		# basic check for non empty; not set
		if [[ ! -z "$value" && "$value" != "Not set" ]]; then
			values[$i]=$(echo $value | cut -d'|' -f2)
		fi
	done

	# all parameters found and valid?
	if [ "${#values[@]}" -lt "4" ]; then
		echo "${LOG_DEBUG}found ${#values[*]} parameters expected 4"
		return
	fi

	if [ "${values[0]}" != "configured" ]; then
		return
	fi

	if [ "${values[3]}" != "accesspoint" ]; then
		return
	fi

	which hostapd &> /dev/null
	if [ $? -ne 0 ]; then
		echo "${LOG_CRIT}hostapd not installed"
		return
	fi

	which dnsmasq &> /dev/null
	if [ $? -ne 0 ]; then
		echo "${LOG_CRIT}dnsmasqd not installed"
		return
	fi

	AP_NAME=${values[1]}
	AP_PW=${values[2]}

	if [ -f "$HOSTAPD_CONF" ]; then
		# check from config
		OLD_AP_NAME=$(cat $HOSTAPD_CONF | grep "^ssid" | cut -d'=' -f2)
		OLD_AP_PW=$(cat $HOSTAPD_CONF | grep "^wpa_passphrase" | cut -d'=' -f2)
	fi

	# check that hostapd status is not exited
	if [[ $(systemctl status hostapd | grep "(exited)") != "" ]]; then
		echo "${LOG_DEBUG}hostapd status: exited - restarting hostapd"
		systemctl restart hostapd
		wait $!
	fi

	# reconfigure if parameter changed or services aren't started
	if [[ $AP_NAME = $OLD_AP_NAME ]] && [[ $AP_PW = $OLD_AP_PW ]]; then

		echo "${LOG_DEBUG}config is up to date"
		systemctl -q is-active hostapd
		if [ $? -eq 0 ]; then
			echo "${LOG_DEBUG}hostapd is running"
			systemctl -q is-active dnsmasq
			if [ $? -eq 0 ]; then
				echo "${LOG_DEBUG}dnsmasq is running"
				return
			fi
		fi
	fi # reconfigure if parameter changed

	echo "${LOG_DEBUG}configuring WiFi"

	## create hostapd conf
	echo "# generated by deCONZ
interface=wlan0
#driver=nl80211

# AP configuration
ssid=$AP_NAME
channel=11
hw_mode=g
wmm_enabled=1
country_code=DE
ieee80211d=1
ignore_broadcast_ssid=0
auth_algs=1

# AP security
wpa=2
wpa_key_mgmt=WPA-PSK
rsn_pairwise=CCMP
wpa_passphrase=$AP_PW" > $HOSTAPD_CONF

	## Add conf to hostapd daemon
	sed -i '/DAEMON_CONF=/c\DAEMON_CONF=\"\/etc\/hostapd\/hostapd.conf\"' /etc/default/hostapd

	## Static address for access point
	if [[ ! $(grep "interface wlan0" $DHCPCD_CONF) ]]
	then
		echo "# added by deCONZ
interface wlan0
	static ip_address=192.168.8.1/24" >> $DHCPCD_CONF
	fi

	## Add wlan0 interface
	echo "# generated by deCONZ
source-directory /etc/network/interfaces.d

allow-hotplug wlan0" > $INTERFACES

	## Configure dns server
	echo "# generated by deCONZ
interface=wlan0
no-dhcp-interface=lo,eth0
dhcp-range=192.168.8.50,192.168.8.150,24h
dhcp-option=option:dns-server,192.168.8.1
address=/phoscon.app/192.168.8.1
address=/phoscon.lan/192.168.8.1" > $DNSMASQ_CONF

	## restart network
	echo "${LOG_DEBUG}restarting network"
	#ifdown wlan0
	ip link set wlan0 down
	wait $!
	#ifup wlan0
	ip link set wlan0 up
	wait $!

	systemctl daemon-reload
	wait $!

	systemctl restart networking
	wait $!

	sleep 5

	systemctl restart dhcpcd
	wait $!

	sleep 10

	systemctl restart dnsmasq
	wait $!

	sleep 5

	systemctl restart hostapd
	wait $!

	sleep 3

	# check that hostapd status is not exited
	if [[ $(systemctl status hostapd | grep "(exited)") != "" ]]; then
		echo "${LOG_DEBUG}hostapd status: exited - restarting hostapd"
		systemctl restart hostapd
		wait $!
		sleep 3
	fi

	ip link show wlan0 up

	if [ $? -ne 0 ]; then
		echo "${LOG_ALERT}wlan0 interface not up, something went wrong"
	fi
}

while [ 1 ]
do
	checkWifiConfig
	sleep 30
done
